
// *************************************************************************
//    SockStream.h
//
//    The SockStream Class - A class for providing low-level
//                           socket operations.
//
// *************************************************************************

#ifndef _SOCKSTREAM_H_
#define _SOCKSTREAM_H_

#include <iostream.h>

// *************************************************************************

        // default PORT
#define URL_PORT         80

#define HOSTNAME_LENGTH 200

// *************************************************************************

class sockbuf : public streambuf {

  public:

      // used to create a local socket at a specific port
    sockbuf(unsigned short Port);

      // used to create a socket connected to a remote machine 
    sockbuf(char *HostName, unsigned short Port = URL_PORT);

      // create a socket using defaults, or envirornment variables,
      // or the given file descriptor (connect to a remote machine)
    sockbuf(int _sockfd = -1);

   ~sockbuf();

    int connect(char *HostName, unsigned short Port);
    int close();
    int shutdown();

    int   sync();	// only flushes output

      // Check if this socket is valid
    int   valid() { return(_socketfd >= 0); }

    int   socketfd() { return(_socketfd); }

      // Accept or Refuse a connection
    sockbuf     *AcceptConnection();
    void         RefuseConnection();

    char *hostname() { return(host); }
    unsigned short my_port() { return(port); }

  protected:

    int   overflow(int c = EOF);
    int   underflow();
    int   doallocate();

    void  init();
    
  private:

    int   _socketfd;               // The socket's file descriptor
    char  host[HOSTNAME_LENGTH];
    unsigned short port;         

    int get_buffer_size;
    int put_buffer_size;

    sockbuf();
    sockbuf(const sockbuf& other);
    sockbuf& operator=(const sockbuf& other);
};


class SockStream : public iostream {

  public:

    SockStream(streambuf *sb);
   ~SockStream();

      // used to create a local server at a specific port
    SockStream *OpenServer(unsigned short Port);

      // used to create a stream connected to a remote machine 
    SockStream *OpenClient(char *HostName, unsigned short Port = URL_PORT);

    int gets(char *buff, int n);
};

// *************************************************************************

#endif  _SOCKSTREAM_H_
